//=============================================================================
/*:
* @help 
* ================================
*           Emotion Targeting
* ================================
* 
* Put this text in the notes of the skill that you want to exploit emotion, with x being the % chance of the skill targeting someone the user has emotion advantage against.
* <Target: x EXPLOIT EMOTION>
*
* Makes a skill have a chance to target an opponent who has any emotion.
* <Target: x TARGET EMOTION>
*
* Makes a skill have a chance to target someone the user has emotion disadvantage against
* <Target: x TARGET DISADVANTAGE>
*
* 
* Example:
* <Target: 75 EXPLOIT EMOTION>
* This would give the skill a 75% chance of targeting someone the user has emotion advantage against.
*
* If x is 0, the % chance will instead be equal to the user's M.Attack stat, making it possible to change the target chance mid-battle.
* Note that the % chance is only for if the skill tries to use the new targeting, the random targeting can still hit anyone.
*
* ================================
*           Forced Actions
* ================================
* Use notetags from this section to target an opponent whose stat is the highest or lowest.
* These notetags will only have an effect when the skill is forced via a battle event, state, or common event.
* Replace x with + to target the opponent with the highest stat, or - to target the lowest stat.
* 
* Targets the opponent with the highest or lowest maximum HP
* <Target: x FORCE TARGET MHP>
*
* Targets the opponent with the highest or lowest HP
* <Target: x FORCE TARGET HP>
*
* Targets the opponent with the highest or lowest HP Percentage
* <Target: x FORCE TARGET HPPERCENT>
*
* Targets the opponent with the highest or lowest maximum MP
* <Target: x FORCE TARGET MMP>
*
* Targets the opponent with the highest or lowest MP
* <Target: x FORCE TARGET MP>
*
* Targets the opponent with the highest or lowest MP Percentage
* <Target: x FORCE TARGET MPPERCENT>
*
* Targets the opponent with the highest or lowest ATK
* <Target: x FORCE TARGET ATK>
*
* Targets the opponent with the highest or lowest DEF
* <Target: x FORCE TARGET DEF>
*
* Targets the opponent with the highest or lowest AGI
* <Target: x FORCE TARGET AGI>
*
* Targets the opponent with the highest or lowest LUK
* <Target: x FORCE TARGET LUK>
*
*
* Example:
* <Target: + TARGET MHP>
* When this skill is forced, it will target the opponent with the highest max HP.
*/
//=============================================================================

let old_Game_Action_prototype_needsSelection = Game_Action.prototype.needsSelection;
Game_Action.prototype.needsSelection = function () {
    // If Action is for everyone or targets a type of emotion
    if (this.isForEverybody() || this.isExploitEmotion() || this.isTargetEmotion() || this.isTargetMHP() || this.isTargetHP() || this.isTargetHpPercent() || this.isTargetMMP() || this.isTargetMP() || this.isTargetMpPercent() || this.isTargetATK() || this.isTargetDEF() || this.isTargetAGI() || this.isTargetLUK() || this.isTargetDisadvantage()) { return false; };
    // Return Original Function
    return _TDS_.OmoriBattleSystem.Game_Action_needsSelection.call(this);
};
//=============================================================================
// * Determine if Action can exploit emotions
//=============================================================================
Game_Action.prototype.isExploitEmotion = function () {
    // Get Item
    let item = this.item();
    // If item exist
    if (item) {
        if (item.meta.Target && item.meta.Target.match(/(\d+)[ ]EXPLOIT EMOTION/i)) {
            targetVar = parseInt(RegExp.$1);
            this.scope = 12;
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Action targets foes with emotion
//=============================================================================
Game_Action.prototype.isTargetEmotion = function () {
    // Get Item
    let item = this.item();
    // If item exist
    if (item) {
        if (item.meta.Target && item.meta.Target.match(/(\d+)[ ]TARGET EMOTION/i)) {
            targetVar = parseInt(RegExp.$1);
            this.scope = 12;
            return true;
        }
    };
    // Return false by default
    return false;
};


//=============================================================================
// * Determine if Action targets foe the user has disadvantage against
//=============================================================================
Game_Action.prototype.isTargetDisadvantage = function () {
    // Get Item
    let item = this.item();
    // If item exist
    if (item) {
        if (item.meta.Target && item.meta.Target.match(/(\d+)[ ]TARGET DISADVANTAGE/i)) {
            targetVar = parseInt(RegExp.$1);
            this.scope = 12;
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on max HP
//=============================================================================
Game_Action.prototype.isTargetMHP = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]\bFORCE TARGET MHP\b/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest max HP");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest max HP");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on HP
//=============================================================================
Game_Action.prototype.isTargetHP = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]\bFORCE TARGET HP\b/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest HP");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest HP");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on HP Percentage
//=============================================================================
Game_Action.prototype.isTargetHpPercent = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET HPPERCENT/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest HP PERCENT");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest HP PERCENT");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on max MP
//=============================================================================
Game_Action.prototype.isTargetMMP = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]\bFORCE TARGET MMP\b/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest max MP");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest max MP");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on MP
//=============================================================================
Game_Action.prototype.isTargetMP = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]\bFORCE TARGET MP\b/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest MP");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest MP");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on MP Percentage
//=============================================================================
Game_Action.prototype.isTargetMpPercent = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET MPPERCENT/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest MP PERCENT");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest MP PERCENT");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on ATK
//=============================================================================
Game_Action.prototype.isTargetATK = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET ATK/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest ATK");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest ATK");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on DEF
//=============================================================================
Game_Action.prototype.isTargetDEF = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET DEF/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest DEF");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest DEF");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on AGI
//=============================================================================
Game_Action.prototype.isTargetAGI = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET AGI/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest AGI");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest AGI");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Determine if Forced Action targets foes based on LUK
//=============================================================================
Game_Action.prototype.isTargetLUK = function () {
    // Get Item
    let item = this.item();
    // If item exists and was forced
    if (item && this._forcing) {
        if (item.meta.Target && item.meta.Target.match(/(\+|\-)[ ]FORCE TARGET LUK/i)) {
            this.scope = 12;
            if (item.meta.Target.includes("+")) {
                targetVar = 1;
                console.log("Targeting highest LUK");
            } else if (item.meta.Target.includes("-")) {
                targetVar = 0;
                console.log("Targeting lowest LUK");
            } else {
                return false;
            }
            return true;
        }
    };
    // Return false by default
    return false;
};

//=============================================================================
// * Make Targets
//=============================================================================
let old_Game_Action_prototype_makeTargets = Game_Action.prototype.makeTargets
Game_Action.prototype.makeTargets = function () {
    if (this.isValid() && this.isTauntable()) {
        $gameTemp._taunt = true
        $gameTemp._tauntAction = this
    }
    foes = this.opponentsUnit().aliveMembers();
    if (this.isValid() && this.isTauntable()) {
        $gameTemp._taunt = false
    }
    $gameTemp._tauntAction = undefined
    var targets = [];
    // If action is for everyone
    if (this.isForEverybody()) {
        // Initialize Target Array
        let targets = this.friendsUnit().aliveMembers().concat(this.opponentsUnit().aliveMembers());
        // Return Targets
        return this.repeatTargets(targets);
    };
    let action = BattleManager._subject.currentAction();
    if (this.isForOpponent()) {
        //=============================================================================
        // * If action can exploit emotion
        //=============================================================================
        if (this.isExploitEmotion()) {
            var targetVar2 = ((Math.random() * 99) + 1);
            if (targetVar == 0) { targetVar = BattleManager._subject.mat };
            console.log("targetVar = " + targetVar);
            console.log("targetVar2 = " + targetVar2);
            //Chance of exploiting emotion
            if (targetVar2 <= targetVar) {
                console.log("Emotion was exploited");
                if (BattleManager._subject.isEmotionAffected("happy")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('ANGRY') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                } else if (BattleManager._subject.isEmotionAffected("sad")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('HAPPY') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                } else if (BattleManager._subject.isEmotionAffected("angry")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('SAD') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                }
            } else {
                console.log("Emotion was not exploited");
            }
            if (targets.length == 0) {
                for (var i = 0; i < foes.length; ++i) {
                    var member = foes[i];
                    targets.push(member);
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action can target emotion
        //=============================================================================
        if (this.isTargetEmotion()) {
            var targetVar2 = ((Math.random() * 99) + 1);
            if (targetVar == 0) { targetVar = BattleManager._subject.mat };
            console.log("targetVar = " + targetVar);
            console.log("targetVar2 = " + targetVar2);
            //Chance of exploiting emotion
            if (targetVar2 <= targetVar) {
                console.log("Emotion was targeted");
                for (var i = 0; i < foes.length; ++i) {
                    var member = foes[i];
                    if (member.isStateCategoryAffected('EMOTION')) targets.push(member);
                }
            } else {
                console.log("Emotion was not targeted");
            }
            if (targets.length == 0) {
                for (var i = 0; i < foes.length; ++i) {
                    var member = foes[i];
                    targets.push(member);
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action can target emotion disadvantage
        //=============================================================================
        if (this.isTargetDisadvantage()) {
            var targetVar2 = ((Math.random() * 99) + 1);
            if (targetVar == 0) { targetVar = BattleManager._subject.mat };
            console.log("targetVar = " + targetVar);
            console.log("targetVar2 = " + targetVar2);
            //Chance of exploiting emotion
            if (targetVar2 <= targetVar) {
                console.log("Emotion was exploited");
                if (BattleManager._subject.isEmotionAffected("happy")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('SAD') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                } else if (BattleManager._subject.isEmotionAffected("sad")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('ANGRY') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                } else if (BattleManager._subject.isEmotionAffected("angry")) {
                    for (var i = 0; i < foes.length; ++i) {
                        var member = foes[i];
                        if (member.isStateCategoryAffected('HAPPY') || member.isStateCategoryAffected('AFRAID') || member.isStateCategoryAffected('PANIC')) targets.push(member);
                    }
                }
            } else {
                console.log("Emotion disadvantage was not targeted");
            }
            if (targets.length == 0) {
                for (var i = 0; i < foes.length; ++i) {
                    var member = foes[i];
                    targets.push(member);
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on max HP
        //=============================================================================
        if (this.isTargetMHP()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest max HP
                    if (member.mhp > targetStat) {
                        targetStat = member.mhp
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest max HP
                    if (member.mhp < targetStat) {
                        targetStat = member.mhp
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on HP
        //=============================================================================
        if (this.isTargetHP()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest HP
                    if (member.hp > targetStat) {
                        targetStat = member.hp
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest HP
                    if (member.hp < targetStat) {
                        targetStat = member.hp
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on HP Percentage
        //=============================================================================
        if (this.isTargetHpPercent()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest HP PERCENT
                    if ((member.hp / member.mhp) >= targetStat) {
                        targetStat = (member.hp / member.mhp);
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest max HP PERCENT
                    if ((member.hp / member.mhp) <= targetStat) {
                        targetStat = (member.hp / member.mhp);
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on max MP
        //=============================================================================
        if (this.isTargetMMP()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest max MP
                    if (member.mmp > targetStat) {
                        targetStat = member.mmp
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest max MP
                    if (member.mmp < targetStat) {
                        targetStat = member.mmp
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on MP
        //=============================================================================
        if (this.isTargetMP()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest MP
                    if (member.mp > targetStat) {
                        targetStat = member.mp
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest MP
                    if (member.mp < targetStat) {
                        targetStat = member.mp
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on MP Percentage
        //=============================================================================
        if (this.isTargetMpPercent()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest MP PERCENT
                    if ((member.mp / member.mmp) >= targetStat) {
                        targetStat = (member.mp / member.mmp);
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest MP PERCENT
                    if ((member.mp / member.mmp) <= targetStat) {
                        targetStat = (member.mp / member.mmp);
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
        //=============================================================================
        // * If action targets based on ATK
        //=============================================================================
        if (this.isTargetATK()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest ATK
                    if (member.atk > targetStat) {
                        targetStat = member.atk
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest ATK
                    if (member.atk < targetStat) {
                        targetStat = member.atk
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };

        //=============================================================================
        // * If action targets based on DEF
        //=============================================================================
        if (this.isTargetDEF()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest DEF
                    if (member.def > targetStat) {
                        targetStat = member.def
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest DEF
                    if (member.def < targetStat) {
                        targetStat = member.def
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };

        //=============================================================================
        // * If action targets based on AGI
        //=============================================================================
        if (this.isTargetAGI()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest AGI
                    if (member.agi > targetStat) {
                        targetStat = member.agi
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest AGI
                    if (member.agi < targetStat) {
                        targetStat = member.agi
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };

        //=============================================================================
        // * If action targets based on LUK
        //=============================================================================
        if (this.isTargetLUK()) {
            var targetStat = 0;
            if (targetVar != 1) targetStat = 99999999;
            for (var i = 0; i < foes.length; ++i) {
                var member = foes[i];
                if (targetVar == 1) { // Target highest LUK
                    if (member.luk > targetStat) {
                        targetStat = member.luk
                        console.log(targetStat);
                        targets = [];
                        targets.push(member);
                    }
                } else { // Target lowest LUK
                    if (member.luk < targetStat) {
                        targetStat = member.luk
                        targets = [];
                        targets.push(member);
                    }
                }
            }
            var randomMember = targets[Math.floor(Math.random() * targets.length)]
            targets = [];
            targets.push(randomMember);
            return this.repeatTargets(targets);
        };
    }
    // Return Original Function
    return _TDS_.OmoriBattleSystem.Game_Action_makeTargets.call(this);
};